/**
   USB模式，实现USB键盘功能

 * **/
#include "keyboard.h"
#include "ws2812.h"
#include "usbcode.h"
#include "scan.h"
#include "save.h"
#include "keyboard_usb.h"
#include "USB.h"
#include "sleep.h"
#include "USBHIDKeyboard.h"
#include "USBHIDConsumerControl.h"

USBHIDConsumerControl ConsumerControl;
USBHIDKeyboard Keyboard_USB;

//任务设置
#define USB_TASK_CORE 0     //CPU0
#define USB_TASK_PRIORITY 2 //优先级为0
#define USB_TASK_STACK 4096 //堆栈字数 堆栈位宽32
#define USB_TASK_DELAY 4    //任务延时4ms
uint8_t usb_casp_lock_state = 0;
TaskHandle_t USB_TASK_Handle = NULL; //此任务由主任务打开和关闭

static void usbEventCallback(void* arg, esp_event_base_t event_base, int32_t event_id, void* event_data) {
  if (event_base == ARDUINO_USB_HID_KEYBOARD_EVENTS) {
    arduino_usb_hid_keyboard_event_data_t * data = (arduino_usb_hid_keyboard_event_data_t*)event_data;
    switch (event_id) {
      case ARDUINO_USB_HID_KEYBOARD_LED_EVENT:
        usb_casp_lock_state = data->capslock;
#ifdef DEBUG_K
        Serial.printf("HID KEYBOARD LED: NumLock:%u, CapsLock:%u, ScrollLock:%u\n", data->numlock, data->capslock, data->scrolllock);
        Serial.printf("usb_casp_lock_state");
        Serial.println(usb_casp_lock_state);
#endif
        break;
      default:
        break;
    }
  }
}

void usb_task(void *pvParameters)
{
  uint8_t keyboard_old[KEYBOARD_ROW][KEYBOARD_COLUMN] = {};
  ConsumerControl.begin();
  Keyboard_USB.onEvent(usbEventCallback);
  Keyboard_USB.begin();
  USB.begin();
  Keyboard_USB.pressRaw(HID_KEY_CAPS_LOCK);
  Keyboard_USB.releaseRaw(HID_KEY_CAPS_LOCK);
  for (;;)
  {
    if (usb_casp_lock_state == 1)
    {
      Wire.beginTransmission(0x2D); // Address of the STC8G
      Wire.write(0x01); // Value to write
      Wire.endTransmission();
    }
    else
    {
      Wire.beginTransmission(0x2D); // Address of the STC8G
      Wire.write(0x02); // Value to write
      Wire.endTransmission();
    }
    uint8_t keyboard_new[KEYBOARD_ROW][KEYBOARD_COLUMN] = {};
    if (xQueuePeek(KEYBOARD_SATE_QUEUE, &keyboard_new, 10) == pdTRUE)
    {
      if (keyboard_new[Fn_X][Fn_Y] == 1) //判断Fn按下
      {
        for (uint8_t i = 0; i < KEYBOARD_ROW; i++)
        {
          for (uint8_t j = 0; j < KEYBOARD_COLUMN; j++)
          {
            if (keyboard_new[i][j] != keyboard_old[i][j])
            {
              keyboard_old[i][j] = keyboard_new[i][j];
              if (keyboard_new[i][j] == 1)
              {
                usb_fn(1, i, j);
              }
              else if (keyboard_new[i][j] == 0)
              {
                usb_fn(0, i, j);
              }
              usb_press_interval = millis();
            }
          }
        }
      }
      else
      {
        for (uint8_t i = 0; i < KEYBOARD_ROW; i++)
        {
          for (uint8_t j = 0; j < KEYBOARD_COLUMN; j++)
          {
            if (keyboard_new[i][j] != keyboard_old[i][j])
            {
              keyboard_old[i][j] = keyboard_new[i][j];
              if (keyboard_new[i][j] == 1)
              {
                if (USB_KEY_CODE[i][j] != 0)
                {

#ifdef DEBUG_K
                  Serial.print("i: ");
                  Serial.print( i );
                  Serial.print( "\n" );
                  Serial.print("j: ");
                  Serial.print( j );
                  Serial.print( "\n" );
#endif
                  if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
                  {
                    Keyboard_USB.press(USB_KEY_CODE[i][j]);
                  }
                  else
                  {
                    Keyboard_USB.press(USB_KEY_MAC_CODE[i][j]);
                  }
                }
                if (i == CAPS_X && j == CAPS_Y) //大写锁定
                {
                  if (usb_casp_lock_state == 1)
                  {
                    Wire.beginTransmission(0x2D); // Address of the STC8G
                    Wire.write(0x01); // Value to write
                    Wire.endTransmission();
                  }
                  else
                  {
                    Wire.beginTransmission(0x2D); // Address of the STC8G
                    Wire.write(0x02); // Value to write
                    Wire.endTransmission();
                  }
                }
              }
              else if (keyboard_new[i][j] == 0)
              {
                if (USB_KEY_CODE[i][j] != 0)
                {
                  if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
                  {
                    Keyboard_USB.release(USB_KEY_CODE[i][j]);
                  }
                  else
                  {
                    Keyboard_USB.release(USB_KEY_MAC_CODE[i][j]);
                  }
                }
              }
              usb_press_interval = millis();
            }
          }
        }
      }
      if (digitalRead(KEY_ENTER_PIN))
      {
        KEY_ENTER_State = 1;
        Keyboard_USB.press(KEY_USB_RETURN);
      }
      else
      {
        if (KEY_ENTER_State == 1)
        {
          Keyboard_USB.release(KEY_USB_RETURN);
          KEY_ENTER_State = 0;
        }
      }
      if (millis() - usb_press_interval >= USB_PRESS_TIME && USB_PRESS_TIME > 0) //长时间未操作进入睡眠
      {
        board_sleep(); //进入睡眠
      }
    }
#ifdef USB_TASK_DELAY
    vTaskDelay(USB_TASK_DELAY);
#endif
  }
  //  USB_TASK_Handle = NULL;
  //  vTaskDelete(NULL);
}

/**
   name: usb_task_begin

   返回值：
    成功返回1 失败返回0

   功能：
    开启 USB 任务
 * */
uint8_t usb_task_begin()
{
  if (USB_TASK_Handle != NULL)
  {
    return 1;
  }
  else
  {
    BaseType_t xReturned;
    xReturned = xTaskCreatePinnedToCore(usb_task, "usb_task", USB_TASK_STACK, NULL, USB_TASK_PRIORITY, &USB_TASK_Handle, USB_TASK_CORE);
    if (xReturned == pdPASS)
    {
      return 1;
    }
  }
  return 0;
}

/**
   name: usb_task_end

   功能：
    关闭 USB 任务
 **/
void usb_task_end()
{
  if (USB_TASK_Handle != NULL)
  {
    vTaskDelete(USB_TASK_Handle);
    USB_TASK_Handle = NULL;
  }
}


/*
   name: usb_fn

   传参：
   state_ : 按键状态
   i : 按键所在行
   j : 按键所在列

   功能：
   按下Fn键后调用此函数进行按键处理

   内容：
   判断行和列
 * */
void usb_fn(uint8_t state_, uint8_t i, uint8_t j)
{
  if (i <= 0)
  {
    if (state_ == 1)
    {
      if (i == Fn_F1_X && j == Fn_F1_Y)
      {
        ConsumerControl.press(CONSUMER_CONTROL_BRIGHTNESS_DECREMENT);
      }
      else if (i == Fn_F2_X && j == Fn_F2_Y)
      {
        ConsumerControl.press(CONSUMER_CONTROL_BRIGHTNESS_INCREMENT);
      }
      else if (i == Fn_F3_X && j == Fn_F3_Y)
      {
        if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
        {
          ConsumerControl.press(CONSUMER_CONTROL_CALCULATOR);
        }
        else
        {
          Keyboard_USB.press(KEY_LEFT_CTRL);
          Keyboard_USB.press(KEY_UP_ARROW);
          Keyboard_USB.releaseAll();
        }
      }
      else if (i == Fn_F4_X && j == Fn_F4_Y)
      {
        if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
        {
          ConsumerControl.press(CONSUMER_CONTROL_LOCAL_BROWSER);
        }
        else
        {
          Keyboard_USB.press(KEY_F4);
          Keyboard_USB.releaseAll();
        }
      }
      else if (i == Fn_F5_X && j == Fn_F5_Y)
      {
        if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
        {
          ConsumerControl.press(CONSUMER_CONTROL_CONFIGURATION);
        }
        else
        {

        }
      }
      else if (i == Fn_F6_X && j == Fn_F6_Y)
      {
        if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
        {
          ConsumerControl.press(CONSUMER_CONTROL_SCAN_PREVIOUS);
        }
        else
        {

        }
      }
      else if (i == Fn_F7_X && j == Fn_F7_Y)
      {
        if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
        {
          ConsumerControl.press(CONSUMER_CONTROL_SCAN_NEXT);
        }
        else
        {
          ConsumerControl.press(CONSUMER_CONTROL_SCAN_PREVIOUS);
        }
      }
      else if (i == Fn_F8_X && j == Fn_F8_Y)
      {
        ConsumerControl.press(CONSUMER_CONTROL_PLAY_PAUSE);
      }
      else if (i == Fn_F9_X && j == Fn_F9_Y)
      {
        if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
        {
          ConsumerControl.press(CONSUMER_CONTROL_STOP);
        }
        else
        {
          ConsumerControl.press(CONSUMER_CONTROL_SCAN_NEXT);
        }
      }
      else if (i == Fn_F10_X && j == Fn_F10_Y)
      {
        ConsumerControl.press(CONSUMER_CONTROL_MUTE);
      }
      else if (i == Fn_F11_X && j == Fn_F11_Y)
      {
        ConsumerControl.press(CONSUMER_CONTROL_VOLUME_DECREMENT);
      }
      else if (i == Fn_F12_X && j == Fn_F12_Y)
      {
        ConsumerControl.press(CONSUMER_CONTROL_VOLUME_INCREMENT);
      }
    }
    else if (state_ == 0)
    {
      if (i == Fn_F1_X && j == Fn_F1_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F2_X && j == Fn_F2_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F3_X && j == Fn_F3_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F4_X && j == Fn_F4_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F5_X && j == Fn_F5_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F6_X && j == Fn_F6_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F7_X && j == Fn_F7_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F8_X && j == Fn_F8_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F9_X && j == Fn_F9_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F10_X && j == Fn_F10_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F11_X && j == Fn_F11_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_F12_X && j == Fn_F12_Y)
      {
        ConsumerControl.release();
      }
      else if (i == Fn_ESC_X && j == Fn_ESC_Y) //切换复位灯光
      {
#ifdef DEBUG_K
        Serial.println("USB复位灯光");
#endif
        Keyboard_Config.L_mode = 0;
        set_light_mode(Keyboard_Config.L_mode);
        save_config();
      }
    }
  }
  else
  {
    fn_set(state_, i, j);
  }
}
